@@ -16,6 +16,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+#include "gdk-pixbuf-alias.h"
 #include "gdk-pixdata.h"
 
 #include "gdk-pixbuf-private.h"
@@ -288,6 +290,13 @@ rl_encode_rgbx (guint8 *bp,	/* dest buffer */
   return bp;
 }
 
+/* Used as the destroy notification function for gdk_pixbuf_new() */
+static void
+free_buffer (guchar *pixels, gpointer data)
+{
+	g_free (pixels);
+}
+
 /**
  * gdk_pixdata_from_pixbuf:
  * @pixdata: a #GdkPixdata to fill.
@@ -326,16 +335,36 @@ gdk_pixdata_from_pixbuf (GdkPixdata      *pixdata,
     {
       guint pad, n_bytes = rowstride * height;
       guint8 *img_buffer_end, *data;
+      GdkPixbuf *buf = NULL;
 
+      if (n_bytes % bpp != 0) 
+	{
+	  rowstride = pixbuf->width * bpp;
+	  n_bytes = rowstride * height;
+	  data = g_malloc (n_bytes);
+	  buf = gdk_pixbuf_new_from_data (data,
+					  GDK_COLORSPACE_RGB,
+					  pixbuf->has_alpha, 8,
+					  pixbuf->width,
+					  pixbuf->height,
+					  rowstride,
+					  free_buffer, NULL);
+	  gdk_pixbuf_copy_area (pixbuf, 0, 0, pixbuf->width, pixbuf->height,
+				buf, 0, 0);
+	}
+      else
+	buf = (GdkPixbuf *)pixbuf;
       pad = rowstride;
       pad = MAX (pad, 130 + n_bytes / 127);
       data = g_new (guint8, pad + n_bytes);
       free_me = data;
       img_buffer = data;
       img_buffer_end = rl_encode_rgbx (img_buffer,
-				       pixbuf->pixels, pixbuf->pixels + n_bytes,
+				       buf->pixels, buf->pixels + n_bytes,
 				       bpp);
       length = img_buffer_end - img_buffer;
+      if (buf != pixbuf)
+	g_object_unref (buf);
     }
   else
     {
@@ -399,7 +428,9 @@ gdk_pixbuf_from_pixdata (const GdkPixdata *pixdata,
 	{
 	  g_set_error (error, GDK_PIXBUF_ERROR,
 		       GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-		       _("failed to allocate image buffer of %u bytes"),
+		       ngettext("failed to allocate image buffer of %u byte",
+				"failed to allocate image buffer of %u bytes",
+				pixdata->rowstride * pixdata->height),
 		       pixdata->rowstride * pixdata->height);
 	  return NULL;
 	}
@@ -863,7 +894,7 @@ gdk_pixdata_to_csource (GdkPixdata        *pixdata,
  * program, it's probably safe to ignore errors and disable length checks, 
  * since things will always succeed:
  * <informalexample><programlisting>
- * pixbuf = gdk_pixbuf_new_from_inline (-1, myimage_inline, NULL);
+ * pixbuf = gdk_pixbuf_new_from_inline (-1, myimage_inline, FALSE, NULL);
  * </programlisting></informalexample>
  *
  * For non-const inline data, you could get out of memory. For untrusted 
