@@ -1,6 +1,7 @@
 /* GDK - The GIMP Drawing Kit
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
- * Copyright (C) 1998-2002 Tor Lillqvist
+ * Copyright (C) 1998-2004 Tor Lillqvist
+ * Copyright (C) 2001-2004 Hans Breuer
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -105,6 +106,13 @@ static void gdk_win32_draw_glyphs    (GdkDrawable      *drawable,
 				      gint              x,
 				      gint              y,
 				      PangoGlyphString *glyphs);
+static void gdk_win32_draw_glyphs_transformed (GdkDrawable      *drawable,
+					     GdkGC            *gc,
+					     PangoMatrix      *matrix,
+					     PangoFont        *font,
+					     gint              x,
+					     gint              y,
+					     PangoGlyphString *glyphs);
 static void gdk_win32_draw_image     (GdkDrawable     *drawable,
 				      GdkGC           *gc,
 				      GdkImage        *image,
@@ -181,6 +189,7 @@ gdk_drawable_impl_win32_class_init (GdkDrawableImplWin32Class *klass)
   drawable_class->draw_segments = gdk_win32_draw_segments;
   drawable_class->draw_lines = gdk_win32_draw_lines;
   drawable_class->draw_glyphs = gdk_win32_draw_glyphs;
+  drawable_class->draw_glyphs_transformed = gdk_win32_draw_glyphs_transformed;
   drawable_class->draw_image = gdk_win32_draw_image;
   
   drawable_class->set_colormap = gdk_win32_set_colormap;
@@ -612,7 +621,7 @@ generic_draw (GdkDrawable    *drawable,
        * the areas where mask is one. (It is filled with said pattern.)
        */
 
-      if (IS_WIN_NT ())
+      if (G_WIN32_IS_NT_BASED ())
 	{
 	  GDI_CALL (MaskBlt, (hdc, region->extents.x1, region->extents.y1,
 			      width, height,
@@ -727,7 +736,7 @@ draw_rectangle (GdkGCWin32 *gcwin32,
   x -= x_offset;
   y -= y_offset;
 
-  if (!filled && gcwin32->pen_dashes && !IS_WIN_NT ())
+  if (!filled && gcwin32->pen_dashes && !G_WIN32_IS_NT_BASED ())
     {
       render_line_vertical (hdc, x, y, y+height+1,
 			    gcwin32->pen_width,
@@ -802,12 +811,9 @@ draw_arc (GdkGCWin32 *gcwin32,
 {
   HGDIOBJ old_pen;
   gboolean filled;
-  gint x;
-  gint y;
-  gint width;
-  gint height;
-  gint angle1;
-  gint angle2;
+  gint x, y;
+  gint width, height;
+  gint angle1, angle2;
   int nXStartArc, nYStartArc, nXEndArc, nYEndArc;
 
   filled = va_arg (args, gboolean);
@@ -827,18 +833,17 @@ draw_arc (GdkGCWin32 *gcwin32,
     }
   else if (angle2 > 0)
     {
-      /* The 100. is just an arbitrary value */
-      nXStartArc = x + width/2 + 100. * cos(angle1/64.*2.*G_PI/360.);
-      nYStartArc = y + height/2 + -100. * sin(angle1/64.*2.*G_PI/360.);
-      nXEndArc = x + width/2 + 100. * cos((angle1+angle2)/64.*2.*G_PI/360.);
-      nYEndArc = y + height/2 + -100. * sin((angle1+angle2)/64.*2.*G_PI/360.);
+      nXStartArc = x + width/2 + width * cos(angle1/64.*2.*G_PI/360.);
+      nYStartArc = y + height/2 + -height * sin(angle1/64.*2.*G_PI/360.);
+      nXEndArc = x + width/2 + width * cos((angle1+angle2)/64.*2.*G_PI/360.);
+      nYEndArc = y + height/2 + -height * sin((angle1+angle2)/64.*2.*G_PI/360.);
     }
   else
     {
-      nXEndArc = x + width/2 + 100. * cos(angle1/64.*2.*G_PI/360.);
-      nYEndArc = y + height/2 + -100. * sin(angle1/64.*2.*G_PI/360.);
-      nXStartArc = x + width/2 + 100. * cos((angle1+angle2)/64.*2.*G_PI/360.);
-      nYStartArc = y + height/2 + -100. * sin((angle1+angle2)/64.*2.*G_PI/360.);
+      nXEndArc = x + width/2 + width * cos(angle1/64.*2.*G_PI/360.);
+      nYEndArc = y + height/2 + -height * sin(angle1/64.*2.*G_PI/360.);
+      nXStartArc = x + width/2 + width * cos((angle1+angle2)/64.*2.*G_PI/360.);
+      nYStartArc = y + height/2 + -height * sin((angle1+angle2)/64.*2.*G_PI/360.);
     }
   
   if (filled)
@@ -906,6 +911,7 @@ draw_polygon (GdkGCWin32 *gcwin32,
 {
   gboolean filled;
   POINT *pts;
+  HPEN old_pen;
   gint npoints;
   gint i;
 
@@ -921,7 +927,14 @@ draw_polygon (GdkGCWin32 *gcwin32,
       }
 
   if (filled)
-    GDI_CALL (Polygon, (hdc, pts, npoints));
+    {
+      old_pen = SelectObject (hdc, GetStockObject (NULL_PEN));
+      if (old_pen == NULL)
+	WIN32_GDI_FAILED ("SelectObject");
+      GDI_CALL (Polygon, (hdc, pts, npoints));
+      if (old_pen != NULL)
+	GDI_CALL (SelectObject, (hdc, old_pen));
+    }
   else
     GDI_CALL (Polyline, (hdc, pts, npoints));
 }
@@ -1028,7 +1041,7 @@ gdk_win32_draw_text (GdkDrawable *drawable,
 {
   const GdkGCValuesMask mask = GDK_GC_FOREGROUND|GDK_GC_FONT;
   wchar_t *wcstr, wc;
-  gint wlen;
+  glong wlen;
   gdk_draw_text_arg arg;
 
   if (text_length == 0)
@@ -1054,15 +1067,11 @@ gdk_win32_draw_text (GdkDrawable *drawable,
     }
   else
     {
-      wcstr = g_new (wchar_t, text_length);
-      if ((wlen = _gdk_utf8_to_ucs2 (wcstr, text, text_length, text_length)) == -1)
-	g_warning ("gdk_win32_draw_text: _gdk_utf8_to_ucs2 failed");
-      else
-	_gdk_wchar_text_handle (font, wcstr, wlen, gdk_draw_text_handler, &arg);
+      wcstr = g_utf8_to_utf16 (text, text_length, NULL, &wlen, NULL);
+      _gdk_wchar_text_handle (font, wcstr, wlen, gdk_draw_text_handler, &arg);
       g_free (wcstr);
     }
 
-
   gdk_win32_hdc_release (drawable, gc, mask);
 }
 
@@ -1175,15 +1184,19 @@ draw_segments (GdkGCWin32 *gcwin32,
   nsegs = va_arg (args, gint);
 
   if (x_offset != 0 || y_offset != 0)
-    for (i = 0; i < nsegs; i++)
-      {
-	segs[i].x1 -= x_offset;
-	segs[i].y1 -= y_offset;
-	segs[i].x2 -= x_offset;
-	segs[i].y2 -= y_offset;
-      }
+    {
+      /* must not modify in place, but could splice in the offset all below */
+      segs = g_memdup (segs, nsegs * sizeof (GdkSegment));
+      for (i = 0; i < nsegs; i++)
+        {
+          segs[i].x1 -= x_offset;
+          segs[i].y1 -= y_offset;
+          segs[i].x2 -= x_offset;
+          segs[i].y2 -= y_offset;
+        }
+    }
 
-  if (gcwin32->pen_dashes && !IS_WIN_NT ())
+  if (gcwin32->pen_dashes && !G_WIN32_IS_NT_BASED ())
     {
       for (i = 0; i < nsegs; i++)
 	{
@@ -1233,18 +1246,32 @@ draw_segments (GdkGCWin32 *gcwin32,
        * e.g. at xpm icons produced with gdk_pixbuf_new_from_xpm_data trough
        * gdk_pixbuf_render_threshold_alpha (testgtk folder icon or
        * Dia's toolbox icons) but only on win9x ... --hb
+       *
+       * Update : see bug #81895 and bug #126710 why this is finally
+       *          needed on any win32 platform ;-)
        */
-      if (gcwin32->pen_width <= 1 && !IS_WIN_NT())
+      if (gcwin32->pen_width <= 1)
         {
           GdkSegment *ps = &segs[nsegs-1];
-          int xc = (ps->y2 == ps->y1) ? 0 : ((ps->x1 < ps->x2) ? 1 : -1);
-          int yc = (ps->x2 == ps->x1) ? 0 : ((ps->y1 < ps->y2) ? 1 : -1);
-	/* don't forget single point lines */
-	xc = (0 == xc && 0 == yc) ? 1 : xc;
+          int xc = 0, yc = 0;
+
+          if (ps->y2 == ps->y1 && ps->x2 == ps->x1)
+            xc = 1; /* just a point */
+          else if (ps->y2 == ps->y1)
+            xc = (ps->x1 < ps->x2) ? 1 : -1; /* advance x only */
+          else if (ps->x2 == ps->x1)
+            yc = (ps->y1 < ps->y2) ? 1 : -1; /* advance y only */
+          else
+            {
+              xc = (ps->x1 < ps->x2) ? 1 : -1;
+              yc = (ps->y1 < ps->y2) ? 1 : -1;
+            }
 
           GDI_CALL (LineTo, (hdc, ps->x2 + xc, ps->y2 + yc));
         }
     }
+  if (x_offset != 0 || y_offset != 0)
+    g_free (segs);
 }
 
 static void
@@ -1311,7 +1338,7 @@ draw_lines (GdkGCWin32 *gcwin32,
 	pts[i].y -= y_offset;
       }
   
-  if (gcwin32->pen_dashes && !IS_WIN_NT ())
+  if (gcwin32->pen_dashes && !G_WIN32_IS_NT_BASED ())
     {
       for (i = 0; i < npoints - 1; i++)
         {
@@ -1421,6 +1448,31 @@ draw_glyphs (GdkGCWin32 *gcwin32,
   pango_win32_render (hdc, font, glyphs, x, y);
 }
 
+static void
+draw_glyphs_transformed (GdkGCWin32 *gcwin32,
+	                   HDC         hdc,
+	                   gint        x_offset,
+	                   gint        y_offset,
+	                   va_list     args)
+{
+  PangoFont *font;
+  gint x;
+  gint y;
+  PangoGlyphString *glyphs;
+  PangoMatrix *matrix;
+
+  matrix = va_arg(args, PangoMatrix *);
+  font = va_arg (args, PangoFont *);
+  x = va_arg (args, gint);
+  y = va_arg (args, gint);
+  glyphs = va_arg (args, PangoGlyphString *);
+
+  x -= x_offset;
+  y -= y_offset;
+
+  pango_win32_render_transformed (hdc, matrix, font, glyphs, x, y);
+}
+
 static void
 gdk_win32_draw_glyphs (GdkDrawable      *drawable,
 		       GdkGC            *gc,
@@ -1447,6 +1499,47 @@ gdk_win32_draw_glyphs (GdkDrawable      *drawable,
   gdk_region_destroy (region);
 }
 
+static void 
+gdk_win32_draw_glyphs_transformed (GdkDrawable      *drawable,
+					     GdkGC            *gc,
+					     PangoMatrix      *matrix,
+					     PangoFont        *font,
+					     gint              x,
+					     gint              y,
+					     PangoGlyphString *glyphs)
+{
+  GdkRectangle bounds;
+  GdkRegion *region;
+  PangoRectangle ink_rect;
+
+  pango_glyph_string_extents (glyphs, font, &ink_rect, NULL);
+
+  bounds.x = x + PANGO_PIXELS (ink_rect.x) - 1;
+  bounds.y = y + PANGO_PIXELS (ink_rect.y) - 1;
+  bounds.width = PANGO_PIXELS (ink_rect.width) + 2;
+  bounds.height = PANGO_PIXELS (ink_rect.height) + 2;
+  region = gdk_region_rectangle (&bounds);
+
+  if (matrix)
+    {
+      /* transform region */
+      bounds.x = bounds.x * matrix->xx + bounds.y * matrix->xy + matrix->x0;
+      bounds.y = bounds.x * matrix->yx + bounds.y * matrix->yy + matrix->x0;
+      bounds.width  = bounds.width  * matrix->xx + bounds.height * matrix->xy;
+      bounds.height = bounds.height * matrix->yx + bounds.width  * matrix->xy;
+ 
+      generic_draw (drawable, gc, GDK_GC_FOREGROUND|GDK_GC_FONT,
+		    draw_glyphs_transformed, region, matrix, font, x, y, glyphs);
+    }
+  else
+    { 
+       generic_draw (drawable, gc, GDK_GC_FOREGROUND|GDK_GC_FONT,
+		    draw_glyphs, region, font, x/PANGO_SCALE, y/PANGO_SCALE, glyphs);
+    }
+
+  gdk_region_destroy (region);
+}
+
 static void
 blit_from_pixmap (gboolean              use_fg_bg,
 		  GdkDrawableImplWin32 *dest,
@@ -1804,3 +1897,15 @@ gdk_win32_drawable_get_handle (GdkDrawable *drawable)
 {
   return GDK_DRAWABLE_HANDLE (drawable);
 }
+
+gboolean
+gdk_draw_rectangle_alpha_libgtk_only (GdkDrawable *drawable,
+				      gint         x,
+				      gint         y,
+				      gint         width,
+				      gint         height,
+				      GdkColor    *color,
+				      guint16      alpha)
+{
+  return FALSE;
+}
