@@ -198,8 +198,8 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @widget: the object which received the signal
    * @prefix: the common prefix of all possible completions
    * 
-   * The ::insert-prefix signal is emitted when the inline autocompletion
-   * is triggered. The default behaviour is to make the entry display the 
+   * Gets emitted when the inline autocompletion is triggered. 
+   * The default behaviour is to make the entry display the 
    * whole prefix and select the newly inserted part.
    *
    * Applications may connect to this signal in order to insert only a
@@ -227,10 +227,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @model: the #GtkTreeModel containing the matches
    * @iter: a #GtkTreeIter positioned at the selected match
    * 
-   * The ::match-selected signal is emitted when a match from the list
-   * is selected. The default behaviour is to replace the contents of the
-   * entry with the contents of the text column in the row pointed to by
-   * @iter.
+   * Gets emitted when a match from the list is selected. 
+   * The default behaviour is to replace the contents of the 
+   * entry with the contents of the text column in the row 
+   * pointed to by @iter.
    *
    * Return value: %TRUE if the signal has been handled
    * 
@@ -252,8 +252,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
    * @widget: the object which received the signal
    * @index: the index of the activated action
    *
-   * The ::action-activated signal is emitted when an action
-   * is activated.
+   * Gets emitted when an action is activated.
    * 
    * Since: 2.4
    */
@@ -301,11 +300,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
                                                      G_PARAM_READWRITE));
 
   /**
-   * GtkEntryCompletion:inline_completion:
+   * GtkEntryCompletion:inline-completion:
    * 
-   * The boolean :inline_completion property determines whether the
-   * common prefix of the possible completions should be inserted 
-   * automatically in the entry.
+   * Determines whether the common prefix of the possible completions 
+   * should be inserted automatically in the entry.
    *
    * Since: 2.6
    **/
@@ -317,10 +315,10 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
  							 FALSE,
  							 G_PARAM_READWRITE));
   /**
-   * GtkEntryCompletion:popup_completion:
+   * GtkEntryCompletion:popup-completion:
    * 
-   * The boolean :popup_completion property determines whether the
-   * possible completions should be shown in a popup window. 
+   * Determines whether the possible completions should be 
+   * shown in a popup window. 
    *
    * Since: 2.6
    **/
@@ -839,12 +837,12 @@ gtk_entry_completion_action_data_func (GtkTreeViewColumn *tree_column,
     return;
 
   if (markup)
-    g_object_set (G_OBJECT (cell),
+    g_object_set (cell,
                   "text", NULL,
                   "markup", string,
                   NULL);
   else
-    g_object_set (G_OBJECT (cell),
+    g_object_set (cell,
                   "markup", NULL,
                   "text", string,
                   NULL);
@@ -954,7 +952,7 @@ gtk_entry_completion_set_model (GtkEntryCompletion *completion,
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (completion->priv->tree_view),
                            GTK_TREE_MODEL (completion->priv->filter_model));
-  g_object_unref (G_OBJECT (completion->priv->filter_model));
+  g_object_unref (completion->priv->filter_model);
 
   if (GTK_WIDGET_VISIBLE (completion->priv->popup_window))
     _gtk_entry_completion_resize_popup (completion);
@@ -1296,9 +1294,8 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
 						  GTK_WIDGET (completion->priv->entry)->window);
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-  width = MIN (completion->priv->entry->allocation.width, monitor.width);
-  gtk_widget_set_size_request (completion->priv->tree_view,
-                               width - 2 * x_border, items * height);
+  width = MIN (completion->priv->entry->allocation.width, monitor.width) - 2 * x_border;
+  gtk_widget_set_size_request (completion->priv->tree_view, width, items * height);
 
   /* default on no match */
   completion->priv->current_selected = -1;
@@ -1308,13 +1305,7 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   if (items)
     {
       gtk_widget_show (completion->priv->action_view);
-
-      gtk_tree_view_column_cell_get_size (gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0),
-                                          NULL, NULL, NULL, NULL,
-                                          &height);
-
-      gtk_widget_set_size_request (completion->priv->action_view,
-                                   width - 2 * x_border, items * height);
+      gtk_widget_set_size_request (completion->priv->action_view, width, -1);
     }
   else
     gtk_widget_hide (completion->priv->action_view);
