@@ -63,7 +63,7 @@ struct _GtkIconFactoryClass
   void (*_gtk_reserved4) (void);
 };
 
-GType           gtk_icon_factory_get_type (void);
+GType           gtk_icon_factory_get_type (void) G_GNUC_CONST;
 GtkIconFactory* gtk_icon_factory_new      (void);
 void            gtk_icon_factory_add      (GtkIconFactory *factory,
                                            const gchar    *stock_id,
@@ -88,11 +88,11 @@ GtkIconSet* gtk_icon_factory_lookup_default  (const gchar     *stock_id);
  * size from the rendered pixbuf, not from here.
  */
 
-#ifndef GTK_MULTIHEAD_SAFE
+#ifndef GDK_MULTIHEAD_SAFE
 gboolean gtk_icon_size_lookup              (GtkIconSize  size,
 					    gint        *width,
 					    gint        *height);
-#endif /* GTK_MULTIHEAD_SAFE */
+#endif /* GDK_MULTIHEAD_SAFE */
 gboolean gtk_icon_size_lookup_for_settings (GtkSettings *settings,
 					    GtkIconSize  size,
 					    gint        *width,
@@ -108,7 +108,7 @@ G_CONST_RETURN gchar* gtk_icon_size_get_name       (GtkIconSize  size);
 
 /* Icon sets */
 
-GType       gtk_icon_set_get_type        (void);
+GType       gtk_icon_set_get_type        (void) G_GNUC_CONST;
 GtkIconSet* gtk_icon_set_new             (void);
 GtkIconSet* gtk_icon_set_new_from_pixbuf (GdkPixbuf       *pixbuf);
 
@@ -135,18 +135,21 @@ void           gtk_icon_set_get_sizes    (GtkIconSet          *icon_set,
                                           GtkIconSize        **sizes,
                                           gint                *n_sizes);
 
-GType          gtk_icon_source_get_type                 (void);
+GType          gtk_icon_source_get_type                 (void) G_GNUC_CONST;
 GtkIconSource* gtk_icon_source_new                      (void);
 GtkIconSource* gtk_icon_source_copy                     (const GtkIconSource *source);
 void           gtk_icon_source_free                     (GtkIconSource       *source);
 
 void           gtk_icon_source_set_filename             (GtkIconSource       *source,
                                                          const gchar         *filename);
+void           gtk_icon_source_set_icon_name            (GtkIconSource       *source,
+                                                         const gchar         *icon_name);
 void           gtk_icon_source_set_pixbuf               (GtkIconSource       *source,
                                                          GdkPixbuf           *pixbuf);
 
-G_CONST_RETURN gchar* gtk_icon_source_get_filename (const GtkIconSource *source);
-GdkPixbuf*            gtk_icon_source_get_pixbuf   (const GtkIconSource *source);
+G_CONST_RETURN gchar* gtk_icon_source_get_filename  (const GtkIconSource *source);
+G_CONST_RETURN gchar* gtk_icon_source_get_icon_name (const GtkIconSource *source);
+GdkPixbuf*            gtk_icon_source_get_pixbuf    (const GtkIconSource *source);
 
 void             gtk_icon_source_set_direction_wildcarded (GtkIconSource       *source,
                                                            gboolean             setting);
