@@ -28,11 +28,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define GTK_TYPE_LIST_STORE	       (gtk_list_store_get_type ())
-#define GTK_LIST_STORE(obj)	       (GTK_CHECK_CAST ((obj), GTK_TYPE_LIST_STORE, GtkListStore))
-#define GTK_LIST_STORE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_LIST_STORE, GtkListStoreClass))
-#define GTK_IS_LIST_STORE(obj)	       (GTK_CHECK_TYPE ((obj), GTK_TYPE_LIST_STORE))
-#define GTK_IS_LIST_STORE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_LIST_STORE))
-#define GTK_LIST_STORE_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_LIST_STORE, GtkListStoreClass))
+#define GTK_LIST_STORE(obj)	       (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_LIST_STORE, GtkListStore))
+#define GTK_LIST_STORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_LIST_STORE, GtkListStoreClass))
+#define GTK_IS_LIST_STORE(obj)	       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_LIST_STORE))
+#define GTK_IS_LIST_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_LIST_STORE))
+#define GTK_LIST_STORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_LIST_STORE, GtkListStoreClass))
 
 typedef struct _GtkListStore       GtkListStore;
 typedef struct _GtkListStoreClass  GtkListStoreClass;
@@ -43,9 +43,9 @@ struct _GtkListStore
 
   /*< private >*/
   gint stamp;
-  gpointer root;
-  gpointer tail;
-  GList *sort_list;
+  gpointer seq;		/* head of the list */
+  gpointer _gtk_reserved1;
+  GList *sort_list;	
   gint n_columns;
   gint sort_column_id;
   GtkSortType order;
@@ -69,7 +69,7 @@ struct _GtkListStoreClass
 };
 
 
-GtkType       gtk_list_store_get_type         (void);
+GType         gtk_list_store_get_type         (void) G_GNUC_CONST;
 GtkListStore *gtk_list_store_new              (gint          n_columns,
 					       ...);
 GtkListStore *gtk_list_store_newv             (gint          n_columns,
@@ -90,7 +90,7 @@ void          gtk_list_store_set              (GtkListStore *list_store,
 void          gtk_list_store_set_valist       (GtkListStore *list_store,
 					       GtkTreeIter  *iter,
 					       va_list       var_args);
-void          gtk_list_store_remove           (GtkListStore *list_store,
+gboolean      gtk_list_store_remove           (GtkListStore *list_store,
 					       GtkTreeIter  *iter);
 void          gtk_list_store_insert           (GtkListStore *list_store,
 					       GtkTreeIter  *iter,
@@ -106,6 +106,19 @@ void          gtk_list_store_prepend          (GtkListStore *list_store,
 void          gtk_list_store_append           (GtkListStore *list_store,
 					       GtkTreeIter  *iter);
 void          gtk_list_store_clear            (GtkListStore *list_store);
+gboolean      gtk_list_store_iter_is_valid    (GtkListStore *list_store,
+                                               GtkTreeIter  *iter);
+void          gtk_list_store_reorder          (GtkListStore *store,
+                                               gint         *new_order);
+void          gtk_list_store_swap             (GtkListStore *store,
+                                               GtkTreeIter  *a,
+                                               GtkTreeIter  *b);
+void          gtk_list_store_move_after       (GtkListStore *store,
+                                               GtkTreeIter  *iter,
+                                               GtkTreeIter  *position);
+void          gtk_list_store_move_before      (GtkListStore *store,
+                                               GtkTreeIter  *iter,
+                                               GtkTreeIter  *position);
 
 
 #ifdef __cplusplus
