@@ -28,6 +28,8 @@
 
 #include <config.h>
 #include <string.h>
+
+#include "gtkalias.h"
 #include "gtkaccellabel.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
@@ -502,7 +504,7 @@ gtk_menu_item_size_request (GtkWidget      *widget,
   g_return_if_fail (requisition != NULL);
 
   gtk_widget_style_get (widget,
- 			"horizontal padding", &horizontal_padding,
+ 			"horizontal_padding", &horizontal_padding,
 			NULL);
   
   bin = GTK_BIN (widget);
@@ -723,6 +725,10 @@ gtk_menu_item_paint (GtkWidget    *widget,
 	  guint horizontal_padding;
 	  GtkTextDirection direction;
 	  GtkArrowType arrow_type;
+	  PangoContext *context;
+	  PangoFontMetrics *metrics;
+	  gint ascent, descent;
+	  PangoLayout *layout;
 
 	  direction = gtk_widget_get_direction (widget);
       
@@ -730,10 +736,17 @@ gtk_menu_item_paint (GtkWidget    *widget,
  				"horizontal_padding", &horizontal_padding,
  				NULL);
  	  
-	  gtk_widget_get_child_requisition (GTK_BIN (menu_item)->child,
-					    &child_requisition);
+	  context = gtk_widget_get_pango_context (GTK_BIN (menu_item)->child);
+	  metrics = pango_context_get_metrics (context, 
+					       GTK_WIDGET (GTK_BIN (menu_item)->child)->style->font_desc,
+					       pango_context_get_language (context));
+	  
+	  ascent = pango_font_metrics_get_ascent (metrics);
+	  descent = pango_font_metrics_get_descent (metrics);
+	  pango_font_metrics_unref (metrics);
+	  
+	  arrow_size = PANGO_PIXELS (ascent + descent) - 2 * widget->style->ythickness;
 
-	  arrow_size = child_requisition.height - 2 * widget->style->ythickness;
 	  arrow_extent = arrow_size * 0.8;
 	  
 	  shadow_type = GTK_SHADOW_OUT;
@@ -825,14 +838,19 @@ gtk_real_menu_item_select (GtkItem *item)
 
   menu_item = GTK_MENU_ITEM (item);
 
-  if (menu_item->submenu)
+  if (menu_item->submenu &&
+      !GTK_WIDGET_MAPPED (menu_item->submenu))
     {
       gint popup_delay;
 
       if (menu_item->timer)
-	g_source_remove (menu_item->timer);
-
-      popup_delay = get_popup_delay (menu_item);
+	{
+	  g_source_remove (menu_item->timer);
+	  menu_item->timer = 0;
+	  popup_delay = 0;
+	}
+      else
+	popup_delay = get_popup_delay (menu_item);
       
       if (popup_delay > 0)
 	{
@@ -962,7 +980,8 @@ gtk_menu_item_select_timeout (gpointer data)
 
   parent = GTK_WIDGET (menu_item)->parent;
 
-  if (parent && GTK_IS_MENU_SHELL (parent) && GTK_MENU_SHELL (parent)->active)
+  if ((GTK_IS_MENU_SHELL (parent) && GTK_MENU_SHELL (parent)->active) || 
+      (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off))
     {
       gtk_menu_item_popup_submenu (data);
       if (menu_item->timer_from_keypress && menu_item->submenu)
