@@ -235,7 +235,7 @@ struct _GtkTextLineDisplay
 {
   PangoLayout *layout;
   GSList *cursors;
-  GSList *shaped_objects;
+  GSList *shaped_objects;	/* Only for backwards compatibility */
   
   GtkTextDirection direction;
 
